<?php
#!/usr/bin/perl
#
# PDFlib Virtual File system (PVF):
# Create a PVF file which holds an image or PDF, and import the data from the
# PVF file
#
# This avoids disk access and is especially useful when the same image or PDF
# is imported multiply. For examples, images which sit in a database don't
# have to be written and re-read from disk, but can be passed to PDFlib
# directly in memory. A similar technique can be used for loading other data
# such as fonts, ICC profiles, etc.
#
# Required software: PDFlib/PDFlib+PDI/PPS 9

/* Some constant SVG data as an example for fetching input data from memory */
$svgdata =
"<?xml version='1.0'?> 
<svg viewBox='0 0 100 100' version='1.1' xmlns='http://www.w3.org/2000/svg'> 
  <circle cx='50' cy='50' r='40' fill='orange'/> 
</svg>";


# This is where the data files are. Adjust as necessary.
$searchpath = dirname(__FILE__,3)."/input";

try {
    # create a new PDFlib object
    $p = new PDFlib();


    $p->set_option("searchpath={" . $searchpath . "}");

    # This means we must check return values of load_font() etc.
    $p->set_option("errorpolicy=return");
    $p->set_option("stringformat=utf8");

    # open new PDF file; insert a file name to create the PDF on disk 
    if ($p->begin_document("", "") == 0) {
        echo("Error: " .  $p->get_errmsg());
        exit(1);
    }

    $p->set_info("Creator", "PDFlib Cookbook");
    $p->set_info("Title", "PDFlib Virtual File System");

    $p->create_pvf("/pvf/svg", $svgdata, "");

    # Load the graphics from the PVF
    $svg = $p->load_graphics("svg", "/pvf/svg", "");
    if ($svg == 0) {
        echo("Error: " .  $p->get_errmsg());
        exit(1);
    }

    # Fit the graphics on page 1
    $p->begin_page_ext(0, 0, "width=a4.width height=a4.height");

    $p->fit_graphics($svg, 350, 750, "");

    $p->end_page_ext("");

    # Fit the graphics on page 2
    $p->begin_page_ext(0, 0, "width=a4.width height=a4.height");

    $p->fit_graphics($svg, 350, 50, "");

    $p->end_page_ext("");

    # Delete the virtual file to free the allocated memory
    $p->delete_pvf("/pvf/svg");

    $p->end_document("");

    $buf = $p->get_buffer();
    $len = strlen($buf);

    header("Content-type: application/pdf");
    header("Content-Length: $len");
    header("Content-Disposition: inline; filename=data_from_memory.pdf");
    print $buf;

}
catch (PDFlibException $e) {
    echo("PDFlib exception occurred:\n" .
        "[" . $e->get_errnum() . "] " . $e->get_apiname() . ": " .
        $e->get_errmsg() . "\n");
    exit(1);
}
catch (Throwable $e) {
    echo($e);
    exit(1);
}

$p = 0;

?>
